$TITLE (iDCM 44 COMMUNICATIONS FIRMWARE: POWER-UP DIAGNOSTICS MODULE)

$DATE (8 AUGUST 86)

$NOMO

$PAGING

$REGISTERBANK(0)



;***************************************************************************

;*

;*     TITLE:          iDCM 44 COMMUNICATIONS FIRMWARE: POWER-UP DIAGNOSTICS

;*

;*     RELEASE:        2.0

;*

;*     DESCRIPTION:    THIS MODULE CONTAINS THE POWER-UP DIAGNOSTICS PROCEDURE

;*

;*     UPDATES:

;*

;***************************************************************************

;*         

;*           COPYRIGHT 1983, 1986 INTEL CORPORATION

;*

;*           INTEL CORPORATION PROPRIETARY INFORMATION.  THIS LISTING

;*           IS SUPPLIED UNDER THE TERMS OF A LICENSE  AGREEMENT WITH

;*           INTEL CORPORATION  AND MAY NOT  BE COPIED NOR  DISCLOSED

;*           EXCEPT IN  ACCORDANCE WITH THE  TERMS OF THAT AGREEMENT.

;* 

;***************************************************************************



NAME    DCM44D



$INCLUDE(REG44.PDF)



USING 0



        CHKSUM          EQU     NOT (02Bh)   ; R2.1 checksum



        MAXADR          EQU     4095

        MAXRAM          EQU     0C0H

        MINRAM          EQU     2H

        MAXMEM          EQU     7FH

        MAXPAGE         EQU     3H

        RED_LED_BIT     EQU     090H

        GRN_LED_BIT     EQU     091H



EXTRN   CODE    (req_init)



DCMFW_DIAG_CODE_SEG             SEGMENT CODE

DCMFW_CODE_FILL_SEG             SEGMENT CODE



;***************************************************************************

;*

;*     SETUP OF RESET VECTOR

;*

;***************************************************************************



CSEG    AT      0



        LJMP    Instruction_Set_Test



;***************************************************************************

;*

;*     SETUP OF CHECK SUM VALUE

;*

;***************************************************************************



CSEG    AT      00FFFH



PUBLIC  dcm_cksum



        dcm_cksum:      DB      CHKSUM



;***************************************************************************

;*

;*     FILLER AREA FOR PROM/ROM

;*

;***************************************************************************



RSEG                    DCMFW_CODE_FILL_SEG



%SET(EMPTY_BYTES, (0fd6h+19h))

%WHILE (%EMPTY_BYTES NE 0FFFH)

  (

  %SET(EMPTY_BYTES, %EMPTY_BYTES+1)



                        DB      00H

  )  





;***************************************************************************

;*

;*     DIAGNOSTIC CODE

;*

;***************************************************************************



RSEG                    DCMFW_DIAG_CODE_SEG



$EJECT

;***************************************************************************

;*

;*     PROCEDURE:      INSTRUCTION_SET_TEST

;*

;*     PURPOSE:        Test Instruction Set Operation

;*

;*     CALLED FROM:    None

;*

;*     ENTERED FROM:   System Reset

;*

;*     CALLS:          None

;*

;*     ENTERS:         ROM_TEST

;*

;*     INPUTS:         None

;*

;*     MODIFIES:       None

;*

;*     DESTROYS:       A

;*                     B

;*

;*     RETURNS:        Nothing

;*

;*     DESCRIPTION:    Tests Instruction Set

;*

;*                     This section ends with the Red LED on and the green

;*                     LED on indicating internal portion of 8044 tests OK

;*

;*     REGISTER BANK:  0

;*

;***************************************************************************



instruction_set_test:



        CLR     GRN_LED_BIT             ;turn off green led

        CLR     RED_LED_BIT             ;turn off red led



        MOV     PSW, #04H               ;Set OV

        MOV     A, #0FFH

        MOV     R0, PSW

        CJNE    R0, #04H, power_up_error

                                        ;This checks parity bit too

        ADD     A, #01H

        ANL     C, /OV

        ANL     C, AC

        JNC     power_up_error          ;Check CY,AC,OV Operation

        ANL     A, #0FFH                ;Check AND Function

        ORL     A, #0AAH                ;Check OR Function

        RLC     A

        RLC     A                       ;A = ABH, CY = 0

        CJNE    A, #0ABH, power_up_error

        JNB     P, power_up_error       ;Check Parity Bit

        RRC     A                       ;A = 55H, CY = 1

        SUBB    A, #50H                 ;Check Subtract, A = 04H

        MOV     B, #12H

        MUL     AB

        MOV     B, #24H

        DIV     AB                      ;A=02, B=00

        CJNE    A, #02H, power_up_error

        MOV     A, B

        CJNE    A, #00H, power_up_error



        SETB    GRN_LED_BIT             ;turn on green led

        SETB    RED_LED_BIT             ;turn on red led



$EJECT

;***************************************************************************

;*

;*     PROCEDURE:      ROM_TEST

;*

;*     PURPOSE:        Validate (P)ROM contents

;*

;*     CALLED FROM:    None

;*

;*     ENTERED FROM:   Instruction_Set_Test

;*

;*     CALLS:          None

;*

;*     ENTERS:         INTERNAL_RAM_TEST

;*

;*     INPUTS:         None

;*

;*     MODIFIES:       A, DPTR

;*

;*     DESTROYS:       DPTR

;*                     A

;*                     R0

;*                     R1

;*                     R2

;*                     R3

;*

;*     RETURNS:        Nothing

;*

;*     DESCRIPTION:    This method gets data for exclusive OR by getting low addr 00 first

;*                     and the going to low addr 0FFH and then 0FEH etc till 01H is reached.

;*                     at this time the upper addr byte is decremented and the process repeated

;*                     on the low address.

;*

;*                     Register Usage:

;*                             DPTR    -       pointer to memory location;*

;*                             A       -       pointer to low byte of DPTR

;*                             R0      -       location to store exclusive or checksum

;*                             R1      -       location to store final DPH value

;*

;*                     This section ends with the Red LED on and the green

;*                     LED off indicating internal portion of 8044 tests OK

;*

;*     REGISTER BANK:  0

;*

;***************************************************************************



rom_test:



        MOV     PSW, #0                 ;Clear PSW and set for register bank

                                        ;  0, flag 0 = 0        

        MOV     DPTR, #MAXADR           ;Initialize to maximum internal

                                        ;   ROM address

        MOV     R0, #00H

        MOV     R1, DPL

        MOV     R2, DPH

;

; EXCLUSIVE OR CHECKSUM LOOP

;

rt_loop:

        CLR     A

        MOV     DPL, R1

        MOV     DPH, R2

        MOVC    A, @A+DPTR              ;get new data byte from ROM

        XRL     A, R0                   ;Exclusive OR new data with checksum

        MOV     R0, A

        DEC     R1

        CJNE    R1, #0FFH, rt_loop      ;Point to next addr.

        DEC     R2                      ;decrement data pointer - high byte

        CJNE    R2, #0FFH, rt_loop      ;Stop after address 0 has been tested

        CJNE    R0, #0FFH, power_up_error

                                        ;Is checksum correct?



        CLR     GRN_LED_BIT             ;turn off green led

        SJMP    internal_RAM_Test

        

$EJECT

;***************************************************************************

;*

;*     PROCEDURE:      Power_Up_Error

;*

;*     PURPOSE:        Halt system after error detected from tests

;*

;*     CALLED FROM:    None

;*

;*     ENTERED FROM:   All

;*

;*     CALLS:          None

;*

;*     ENTERS:         None

;*

;*     INPUTS:         None

;*

;*     MODIFIES:       None

;*

;*     DESTROYS:       Nothing

;*

;*     RETURNS:        Nothing

;*

;*     DESCRIPTION:    Loops

;*

;*     REGISTER BANK:  0

;*

;***************************************************************************



power_up_error:



        SJMP    power_up_error                  ;



$EJECT

;***************************************************************************

;*

;*     PROCEDURE:      INTERNAL_RAM_TEST

;*

;*     PURPOSE:        Test Internal RAM Reliability

;*

;*     CALLED FROM:    None

;*

;*     ENTERED FROM:   ROM_TEST

;*

;*     CALLS:          None

;*

;*     ENTERS:         EXTERNAL_RAM_TEST

;*

;*     INPUTS:         None

;*

;*     MODIFIES:       None

;*

;*     DESTROYS:       DPTR

;*                     A

;*                     R0

;*                     R1

;*                     R2

;*

;*     RETURNS:        Nothing

;*

;*     DESCRIPTION:    This program clears memory, marches a 1 thru

;*                     memory then marches a zero thru memory, except

;*                     for locations 0 - 7.

;*

;*             Register usage

;*                     R0      pointer to byte to be checked and modified at 00H

;*                     R1      new value of the bit to be tested

;*

;*             Memory Usage

;*                     R2      location to store past value of byte under test

;*                     R3      location to store initial byte value 

;*

;*             This section ends with the Red LED off and the green

;*             LED off indicating internal portion of 8044 tests OK

;*

;*     REGISTER BANK:  0

;*

;***************************************************************************



internal_ram_test:



        MOV     R0, #MINRAM             ;set up start addr

        SETB    F0                      ;set flag for zero walk

        CLR     A                       ;set up for walk of zero

        CPL     A

        

irt_walk_zeros:



        CLR     C                       ;set carry to introduce 0



irt_check_byte:



        RLC     A                       ;



        MOV     @R0, A                  ;write new byte to memory

        MOV     AR2, @R0                ;update check word

        CJNE    A, AR2, power_up_error

                                        ;if it has you have an power_up_error

                                        ;Note, CJNE will clear carry when

                                        ;  Equal condition results

        MOV     C, F0                   ;So move the flag bit to C

                                        ;  flag is 0 for ones walk, 1

                                        ;  for zeros walk

; check for finish of rotate of one bit

; If parity is zero, then the one or zero bit has moved into carry.

;  Otherwise, the one or zero bit is still moving.



        JB      P, irt_check_byte



; if one or zero bit has rotated all the way around, then move to next byte



        INC     R0

        CJNE    R0, #MAXRAM, irt_reset_byte

                                        ;Note that this effects carry.

; if address is not the maximum address, then continue

; Otherwise, check the walk test type:



        JNB     F0, irt_done            ;If flag 0 is set, then marches done;

        MOV     R0, #MINRAM             ;set up start addr

        CLR     F0                      ;Otherwise, clear flag for ones test

        CLR     A                       ; set for marching ones, A is 0H



irt_walk_ones:



        SETB    C                       ;Set carry to introduce 1

        SJMP    irt_check_byte



irt_reset_byte:



        JB      F0, irt_walk_zeros      ;If walking zeros, then set

        SJMP    irt_walk_ones



irt_done:



        CLR     RED_LED_BIT             ;Turn off red led



$EJECT

;***************************************************************************

;*

;*     PROCEDURE:      EXTERNAL_RAM_TEST

;*

;*     PURPOSE:        Test Instruction Set Operation

;*

;*     CALLED FROM:    None

;*

;*     ENTERED FROM:   Internal_Ram_Test

;*

;*     CALLS:          None

;*

;*     ENTERS:         RMX51_initialization

;*

;*     INPUTS:         None

;*

;*     MODIFIES:       None

;*

;*     DESTROYS:       DPTR

;*                     A

;*                     R0

;*                     R1

;*                     R2

;*

;*     RETURNS:        Nothing

;*

;*     DESCRIPTION:    This program clears memory, marches a 1 thru memory

;*                     then marches a zero thru memory

;*

;*             Register usage

;*                     DPTR    pointer to memory location

;*                     R0      pointer to low byte of DPTR

;*                     R1      pointer to high byte of DPTR

;*                     R2      register to store initial byte value

;*                     R3      register to store past value of byte under test

;*                     R1      new value of the bit to be tested

;*

;*             Memory Usage

;*                     R2      location to store past value of byte under test

;*                     R3      location to store initial byte value 

;*

;*

;*             This section ends with the Red LED off and the green

;*             LED on indicating internal portion of 8044 tests OK

;*

;*     REGISTER BANK:  0

;*

;***************************************************************************



external_ram_test:



        MOV     DPTR, #0                ;set up start addr

        SETB    F0                      ;set flag for zero walk

        CLR     A                       ;set up for walk of zero

        CPL     A



ert_walk_zeros:



        CLR     C                       ;set carry flag for ones march

                

ert_check_byte:



        RLC     A                       ;



        MOVX    @DPTR, A                ;write new byte to memory

        MOV     R2, A                   ;update check word

        MOVX    A, @DPTR                ;check to see if it has changed

        CJNE    A, AR2, power_up_error  ;if it has you have an power_up_error

                                        ;Note, CJNE will clear carry when

                                        ;  Equal condition results

        MOV     C, F0                   ;So move the flag bit to C

                                        ;  flag is 0 for ones walk, 1

                                        ;  for zeros walk



; check for finish of rotate of one bit

; If parity is zero, then the one or zero bit has moved into carry.

;  Otherwise, the one or zero bit is still moving.



        JB      P, ert_check_byte



; if one bit has rotated all the way around, then move to next byte



        INC     DPL

        MOV     R0, DPL

        CJNE    R0, #0, ert_reset_byte

; if low byte of address is not 0, then continue with same high byte

        INC     DPH

        MOV     R1, DPH                 ;Check for completion of last byte check

        CJNE    R1, #MAXPAGE, ert_reset_byte

                                        ;Note that this effects carry.

; if high byte of address is not the maximum address, then continue

; Otherwise, check the walk test type:



        JNB     F0, ert_done            ;If flag 0 is set, then marches done;

        MOV     DPTR, #0                ;set up start addr

        CLR     F0                      ;Otherwise, clear flag for ones test

        CLR     A                       ; set for marching ones, A is 0H



ert_walk_ones:



        SETB    C                       ;Set carry to introduce 1

        SJMP    ert_check_byte



ert_reset_byte:



        JB      F0, ert_walk_zeros      ;If walking zeros, then set

        SJMP    ert_walk_ones



ert_done:



        SETB    GRN_LED_BIT             ;Turn on green led

        MOV     SP, #7                  ;Set stack pointer to reset value

        LJMP    req_init                ;



END

